/*============================================================================
  File:     Generalizing the Tipping Point.sql

  Summary:  Various examples and ways to see the estimated and actual tipping
			point of when a nonclustered index is not used vs. doing a table
			scan.
  
  Date:     February 2011

  SQL Server Version: SQL Server 2005/2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Finding the *estimated* tipping point for every table:
USE AdventureWorksDW2008
go

SELECT OBJECT_NAME(object_id) AS [TableName]
	, record_count AS [RowCount]
	, page_count
    , page_count * .25 AS LowRangePages
    , page_count * .33 AS HighRangePages
    , ((page_count * .25)/record_count * 100) AS [LowRangeRows (Percent)]
    , ((page_count * .33)/record_count * 100) AS [HighRangeRows (Percent)]
    , * 
FROM sys.dm_db_index_physical_stats
(db_id(), null, null, null, 'sampled')
WHERE index_id IN (0,1) 
AND page_count > 100
ORDER BY [RowCount] DESC
go

-- What is your database's average?
SELECT DB_NAME(), AVG(tp.[LOW]) AS [Avg Low Percent]
	, AVG(tp.[High]) AS [Avg High Percent]
FROM (SELECT ((page_count * .25)/record_count *100) AS [Low]
    , ((page_count * .33)/record_count * 100) AS [High]
	FROM sys.dm_db_index_physical_stats
		(db_id(), null, null, null, 'sampled')
	WHERE index_id IN (0,1) AND page_count > 100) AS tp
go
	
-- What about the average across all databases on your server:
sp_msforeachdb 'USE ?; SELECT DB_NAME(), AVG(tp.[LOW]) AS [Avg Low Percent]
	, AVG(tp.[High]) AS [Avg High Percent]
FROM (SELECT ((page_count * .25)/record_count *100) AS [Low]
    , ((page_count * .33)/record_count * 100) AS [High]
	FROM sys.dm_db_index_physical_stats
		(db_id(), null, null, null, ''sampled'')
	WHERE index_id IN (0,1) AND page_count > 100) AS tp'
go